from datetime import datetime, date
import json

from django.http import JsonResponse
from django.core.urlresolvers import reverse
from django.shortcuts import render, get_object_or_404
from django.http import Http404

from models import Tag, Vote, VoteToTag
from patterns import DAY_SHORT_PATTERN, YEAR_MONTH_PATTERN


def vote_on_day(request, day_string_short):
    success = False

    try:
        vote_value = request.POST['vote_value']
        vote_comment = request.POST['vote_comment']
    except KeyError:
        return render(request, 'RateYourLife/error.html', {
            'error_message': "You didn't vote.",
        })
    else:
        day_date = datetime.strptime(day_string_short, DAY_SHORT_PATTERN).date()
        vote = Vote.objects.get_or_create(day=day_date)[0]
        vote.value = vote_value
        vote.comment = vote_comment.strip()
        vote.save()
        success = True

        return JsonResponse({'success': success})


def get_day_vote(request, day_string_short):
    day_date = datetime.strptime(day_string_short, DAY_SHORT_PATTERN).date()
    try:
        vote = get_object_or_404(Vote, day=day_date)
        return JsonResponse({'message': vote.value})
    except Http404:
        return JsonResponse({'message': ''})


def add_day_tag(request, day_string_short, tag_name):
    success = False

    # get is_positive
    is_positive = request.POST['is_positive']
    # get vote
    day_date = datetime.strptime(day_string_short, DAY_SHORT_PATTERN).date()
    vote = Vote.objects.get_or_create(day=day_date)[0]
    # get tag
    try:
        tag = Tag.objects.get(name=tag_name)
    except Tag.DoesNotExist:
        return render(request, 'RateYourLife/error.html', {
            'error_message': "The tag you tried to add doesn't exist.",
        })

    if is_positive == 'positive' or is_positive == 'negative':
        is_positive_bool = is_positive == 'positive'
        vote_to_tag_queryset = VoteToTag.objects.filter(vote=vote, tag=tag)
        if vote_to_tag_queryset:
            # exists already, only update it
            vote_to_tag = vote_to_tag_queryset[0]
            vote_to_tag.is_positive = is_positive_bool
            vote_to_tag.save()
            success = True
        else:
            # doesn't exist, create it
            VoteToTag.objects.create(vote=vote, tag=tag, is_positive=is_positive_bool)
            success = True
    elif is_positive == 'unused':
        # delete VoteToTag
        vote_to_tag_queryset = VoteToTag.objects.filter(vote=vote, tag=tag)
        if vote_to_tag_queryset:
            vote_to_tag_queryset[0].delete()
            success = True

    return JsonResponse({'success': success})


def calendar_events_source(request, year_month_string):
    if not year_month_string:
        year_month_string = datetime.now().strftime(YEAR_MONTH_PATTERN)

    year_month_date = datetime.strptime(year_month_string, YEAR_MONTH_PATTERN).date()
    vote_to_tags = VoteToTag.objects.filter(vote__day__year=year_month_date.year,
                                            vote__day__month=year_month_date.month)

    event_source_list = []
    for vote_to_tag in vote_to_tags:
        event_class = 'event-success' if vote_to_tag.is_positive else 'event-important'
        event_url = reverse('RateYourLife:tag_detail', args=(vote_to_tag.tag.id,))
        event_start_milliseconds = (vote_to_tag.vote.day - date(1970, 1, 1)).total_seconds() * 1000
        event_end_milliseconds = event_start_milliseconds + 1000
        vote_to_tag_json = """
               {
                    "id": %d,
                    "title": "%s",
                    "url": "%s",
                    "class": "%s",
                    "start": %d,
                    "end": %d
               }
               """ % (
            vote_to_tag.id,
            vote_to_tag.tag.name,
            event_url,
            event_class,
            event_start_milliseconds,
            event_end_milliseconds
        )

        event_source_list.append(vote_to_tag_json)

    event_source_string = "{ \"resultEvents\" : [" + ", ".join(event_source_list) + " ] }"
    event_source_json = json.loads(event_source_string)
    return JsonResponse(event_source_json)
