from django.db import models

from validators import value_between_one_and_ten


class Tag(models.Model):
    name = models.CharField(max_length=40, unique=True)

    def __str__(self):
        return "name %s" % self.name


class Vote(models.Model):
    value = models.PositiveSmallIntegerField(null=True, validators=[value_between_one_and_ten])
    day = models.DateField(unique=True)
    comment = models.CharField(max_length=140, blank=True, default='')
    tags = models.ManyToManyField(Tag, blank=True, through='VoteToTag')

    def __str__(self):
        return "value %s, day %s, comment %s" % (self.value, self.day, self.comment)


class VoteToTag(models.Model):
    vote = models.ForeignKey(Vote)
    tag = models.ForeignKey(Tag)
    is_positive = models.BooleanField(null=False, default=True)

    def __str__(self):
        return "vote %s, tag %s, is_positive %s" % (self.vote, self.tag, self.is_positive)

    class Meta:
        unique_together = ('vote', 'tag',)


